# Generated by Django 4.0.10 on 2023-12-08 16:40

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("mailrelay", "0002_django4_update"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="discordcategory",
            options={"verbose_name_plural": "Discord categories"},
        ),
        migrations.AlterField(
            model_name="relayconfig",
            name="ping_type",
            field=models.CharField(
                choices=[("PN", "(none)"), ("PH", "@here"), ("PE", "@everyone")],
                default="PN",
                help_text="Option to ping every member of the channel.",
                max_length=2,
                verbose_name="channel pings",
            ),
        ),
    ]
